# -*- coding: utf-8 -*-
from z3c.objpath.interfaces import IObjectPath
from zExceptions import NotFound
from zope.component.hooks import getSite
from zope.interface import implementer


@implementer(IObjectPath)
class Zope2ObjectPath(object):
    """Path representation for Zope 2 objects.
    """

    def path(self, obj):
        try:
            return '/'.join(obj.getPhysicalPath())
        except AttributeError:
            raise ValueError(obj)

    def resolve(self, path):
        site = getSite()
        if site is None:
            raise ValueError(path)

        try:
            root = site.getPhysicalRoot()
        except AttributeError:
            raise ValueError(path)

        try:
            return root.unrestrictedTraverse(path)
        except (AttributeError, NotFound, KeyError):
            raise ValueError(path)



# -*- coding: utf-8 -*-
from plone.app.dexterity import _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from plone.supermodel.directives import fieldset
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.interface import provider


@provider(IFormFieldProvider)
class IRelatedItems(model.Schema):
    """Behavior interface to make a Dexterity type support related items.
    """

    relatedItems = RelationList(
        title=_(u'label_related_items', default=u'Related Items'),
        default=[],
        value_type=RelationChoice(
            title=u'Related', vocabulary='plone.app.vocabularies.Catalog'
        ),
        required=False,
    )
    form.widget(
        'relatedItems',
        RelatedItemsFieldWidget,
        vocabulary='plone.app.vocabularies.Catalog',
        pattern_options={
            'recentlyUsed': True  # Just turn on. Config in plone.app.widgets.
        },
    )

    fieldset(
        'categorization', label=_(u'Categorization'), fields=['relatedItems']
    )



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from plone.uuid.interfaces import ATTRIBUTE_NAME
from plone.uuid.interfaces import IUUID
from z3c.relationfield.interfaces import IRelationValue
from zope.component import adapter
from zope.interface import implementer


@implementer(IUUID)
@adapter(IRelationValue)
def rvUUID(context):
    """ Vocabulary validation via p.a.vocabularies CatalogSource
        requires the UUID of the target object to verify membership
    """
    return getattr(aq_base(context.to_object), ATTRIBUTE_NAME, None)



# -*- coding: utf-8 -*-
from z3c.form.datamanager import AttributeField
from z3c.form.datamanager import DictionaryField
from z3c.form.interfaces import NO_VALUE
from z3c.relationfield.interfaces import IRelation
from z3c.relationfield.interfaces import IRelationList
from z3c.relationfield.interfaces import IRelationValue
from z3c.relationfield.relation import RelationValue
from zope.component import adapter
from zope.component import getUtility
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.security.interfaces import ForbiddenAttribute


@adapter(Interface, IRelation)
class RelationDataManager(AttributeField):
    """A data manager which uses the z3c.relationfield api to set
    relationships using a schema field."""

    def get(self):
        """Gets the target"""
        rel = None
        try:
            rel = super(RelationDataManager, self).get()
        except AttributeError:
            # Not set yet
            pass
        if rel is not None:
            if rel.isBroken():
                # XXX: should log or take action here
                return
            return rel.to_object

    def set(self, value):
        """Sets the relationship target"""
        if value is None:
            return super(RelationDataManager, self).set(None)

        current = None
        try:
            current = super(RelationDataManager, self).get()
        except AttributeError:
            pass
        intids = getUtility(IIntIds)
        to_id = intids.getId(value)
        if IRelationValue.providedBy(current):
            # If we already have a relation, just set the to_id
            current.to_id = to_id
        else:
            # otherwise create a relationship
            rel = RelationValue(to_id)
            super(RelationDataManager, self).set(rel)


@adapter(dict, IRelation)
class RelationDictDataManager(DictionaryField):
    """A data manager which uses the z3c.relationfield api to set
    relationships using a schema field, for dict-like contexts."""

    def get(self):
        """Gets the target"""
        rel = None
        try:
            rel = super(RelationDictDataManager, self).get()
        except AttributeError:
            # Not set yet
            pass
        if rel is not None:
            if rel.isBroken():
                # XXX: should log or take action here
                return
            return rel.to_object

    def query(self, default=NO_VALUE):
        """See z3c.form.interfaces.IDataManager"""
        try:
            return self.get()
        except ForbiddenAttribute as e:
            raise e
        except AttributeError:
            if default == NO_VALUE:
                return super(RelationDictDataManager, self).query()
            else:
                return default

    def set(self, value):
        """Sets the relationship target"""
        if value is None:
            return super(RelationDictDataManager, self).set(None)

        current = None
        try:
            current = super(RelationDictDataManager, self).get()
        except AttributeError:
            pass
        intids = getUtility(IIntIds)
        to_id = intids.getId(value)
        if IRelationValue.providedBy(current):
            # If we already have a relation, just set the to_id
            current.to_id = to_id
        else:
            # otherwise create a relationship
            rel = RelationValue(to_id)
            super(RelationDictDataManager, self).set(rel)


@adapter(Interface, IRelationList)
class RelationListDataManager(AttributeField):
    """A data manager which sets a list of relations"""

    def get(self):
        """Gets the target"""
        rel_list = []

        # Calling query() here will lead to infinite recursion!
        try:
            rel_list = super(RelationListDataManager, self).get()

        except AttributeError:
            rel_list = None

        if not rel_list:
            return []

        resolved_list = []
        for rel in rel_list:
            if rel.isBroken():
                # XXX: should log or take action here
                continue
            resolved_list.append(rel.to_object)
        return resolved_list

    def set(self, value):
        """Sets the relationship target"""
        value = value or []
        new_relationships = []
        intids = getUtility(IIntIds)
        for item in value:
            # otherwise create one
            to_id = intids.getId(item)
            new_relationships.append(RelationValue(to_id))
        super(RelationListDataManager, self).set(new_relationships)


@adapter(dict, IRelationList)
class RelationListDictDataManager(DictionaryField):
    """A data manager which sets a list of relations on dictionary"""

    def get(self):
        """Gets the target"""
        rel_list = []

        # Calling query() here will lead to infinite recursion!
        try:
            rel_list = super(RelationListDictDataManager, self).get()
        except AttributeError:
            rel_list = None

        if not rel_list:
            return []

        resolved_list = []
        for rel in rel_list:
            if rel.isBroken():
                # XXX: should log or take action here
                continue
            resolved_list.append(rel.to_object)
        return resolved_list

    def query(self, default=NO_VALUE):
        """See z3c.form.interfaces.IDataManager"""
        try:
            return self.get()
        except ForbiddenAttribute as e:
            raise e
        except AttributeError:
            return default

    def set(self, value):
        """Sets the relationship target"""
        value = value or []
        new_relationships = []
        intids = getUtility(IIntIds)
        for item in value:
            # otherwise create one
            to_id = intids.getId(item)
            new_relationships.append(RelationValue(to_id))
        super(RelationListDictDataManager, self).set(new_relationships)



# -*- coding: utf-8 -*-
from plone.z3cform import layout
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import IFormLayer
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList


class ITestForm(IFormLayer):
    multiple = RelationList(
        title=u'Multiple (Relations field)',
        required=False,
        value_type=RelationChoice(
            title=u'Multiple', vocabulary='plone.app.vocabularies.Catalog'
        ),
    )
    single = RelationChoice(
        title=u'Single',
        required=False,
        vocabulary='plone.app.vocabularies.Catalog',
    )


class TestForm(form.EditForm):
    fields = field.Fields(ITestForm)


TestView = layout.wrap_form(TestForm)
